shinyServer(function(input, output) {

    output$checkGraph <- renderText({
        if(input$whichgraph=="base") return("")
        A <- is.element("ggplot2", installed.packages()[,1])
        B <- is.element("grid", installed.packages()[,1])
        if(A & B) {
            library(ggplot2)
            library(grid)
            return("")
        }    
        if(!A) return(paste(red,"<h4>Warning: ggplot2 graphics package not found, go back to base graphics</h4>"))
        if(!B) return(paste(red,"<h4>Warning: package grid not found, go back to base graphics</h4>"))        
        NULL       
    })
           
   data <- reactive({ 
        if(input$mytabs==1) n <- input$n
        else n <- 1e4
        if(input$dataset=="Pick your own") {
            rdist <- eval(parse(text=paste("r",input$dist,sep="")))
            if(input$npar=="1") x <- rdist(n,input$par1)
            else x <- rdist(n,input$par1,input$par2)
        }    
        if(input$dataset=="Normal")
            x <- rnorm(input$n,input$mu,input$s)
        x
   })
     
  output$plot1 <- renderPlot({
    if(input$mytabs==1) n <- input$n
    else n <- 1e4
    if(input$whichgraph=="base") {
        par(mfrow=c(2,2))
        hist(data(),100,main="",xlab="",ylab="")
        boxplot(data(),horizontal=T)
        qqnorm(data())
        qqline(data())
    }
    else {    
        A <- is.element("ggplot2", installed.packages()[,1])
        B <- is.element("grid", installed.packages()[,1])
        if(A & B) {    
            dta <- data.frame(x=data(),y=rep(0,n))
            plt1 <- ggplot(dta,aes(x))+geom_histogram(binwidth=0.1,color="black",fill="white")
            plt2 <- ggplot(dta,aes(y,x))+geom_boxplot()+ coord_flip()+xlab("")+ylab("")
            plt3 <- qplot(sample = dta$x, stat="qq")
 
            pushViewport(viewport(layout = grid.layout(2, 2)))
            print(plt1, vp = viewport(layout.pos.row = 1, layout.pos.col = 1))
            print(plt2, vp = viewport(layout.pos.row = 1, layout.pos.col = 2))    
            print(plt3, vp = viewport(layout.pos.row = 2, layout.pos.col = 1))    
        }    
    }    
  })
  
  output$plot2 <- renderPlot({
    dat <- data()
    if(length(unique(dat))==length(dat)) {
      xr <- range(dat)
      x <- seq(xr[1],xr[2],length=200)
      if(input$dataset=="Pick your own") {
          ddist <- eval(parse(text=paste("d",input$dist,sep="")))
          if(input$npar=="1") y <- ddist(x,input$par1)
              else y <- ddist(x,input$par1,input$par2)
      }    
      if(input$dataset=="Normal")
          y <- dnorm(x,input$mu,input$s)
      if(input$whichgraph=="base") {
          par(mfrow=c(1,1))    
          plot(x,y,type="l")    
      }
      else {    
        A <- is.element("ggplot2", installed.packages()[,1])
        B <- is.element("grid", installed.packages()[,1])
        if(A & B) {    
             dta <- data.frame(x=x,y=y)
             ggplot(dta,aes(x,y))+geom_line(size=1.2,colour="blue")+xlab("")+ylab("")
        }
      }    
    }
    else {
      x <- as.numeric(names(table(dat)))
      x <- min(x):max(x)
      ddist <- eval(parse(text=paste("d",input$dist,sep="")))
      if(input$npar=="1") y <- ddist(x,input$par1)
      else y <- ddist(x,input$par1,input$par2)
      if(input$whichgraph=="base") {
          par(mfrow=c(1,1))
          plot(x,y,type="n")
          segments(x-0.2,0,x-0.2,y)
          segments(x+0.2,0,x+0.2,y)
          segments(x-0.2,y,x+0.2,y)
          segments(x-0.2,0,x+0.2,0)
      } 
      else {    
        A <- is.element("ggplot2", installed.packages()[,1])
        B <- is.element("grid", installed.packages()[,1])
        if(A & B) {    
             dta <- data.frame(x=x,y=y)
             ggplot(aes(factor(x),weight=y),data=dta)+
             geom_bar(fill="white", colour="black",width=.5)+
             xlab("x")+ylab("Probability")
        }
      }      
    } 
     
  })  
  
  output$appinfo <- renderText({ 
      HTML("<br>&nbsp;<br>&nbsp;<br>&nbsp;<br>
      <a target=\"_blank\" href=\"http://academic.uprm.edu/wrolke/myapps1.htm#distributions\"><h4>What to do</h4></a><br>
      <h6>Written by: Dr. Wolfgang Rolke<br>Created: 11/22/2014</h6>")
      
  })    
})

